@@ -7,7 +7,11 @@ import pandas as pd
 from mondobrain.api import solve_result, solve_start_file
 import mondobrain.datautils as datautils
 from mondobrain.dd_transformer import DDTransformer
-from mondobrain.error import InvalidTargetError, NotEnoughPointsError
+from mondobrain.error import (
+    DataTransformationError,
+    InvalidTargetError,
+    NotEnoughPointsError,
+)
 from mondobrain.metrics import score
 from mondobrain.utils import constants, utilities
 from mondobrain.utils.data import is_discrete
@@ -20,6 +24,7 @@ def solve(
     encode=True,
     sample=True,
     random_state=None,
+    return_diagnostics=False,
     **kwargs,
 ):
     """Run a solve without worrying about API requests.
@@ -103,14 +108,32 @@ def solve(
     result = solve_result(id=task["id"])
 
     rule = result["rule"]
+    diagnostics = result["diagnostics"]
 
     if encode:
-        # We need to decode the rule if we encoded
+        # We need to decode the rule & diagnostics if we encoded
         rule = utilities.decode_rule(rule, encoder)
+        diagnostics = _decode_diagnostics(diagnostics, encoder)
+
+    if return_diagnostics:
+        return rule, diagnostics
 
     return rule
 
 
+def _decode_diagnostics(diagnostics: dict, encoder: DDTransformer) -> dict:
+    diagnostics = diagnostics.copy()
+
+    try:
+        disc_rules = diagnostics["discarded_rules"]["rules"]
+        disc_rules = [utilities.decode_rule(r, encoder) for r in disc_rules]
+        diagnostics["discarded_rules"]["rules"] = disc_rules
+    except KeyError:
+        pass
+
+    return diagnostics
+
+
 def _dimensionality_check(df: pd.DataFrame, outcome: str, target: str):
     col = df[outcome]
 
@@ -129,7 +152,7 @@ def _dimensionality_check(df: pd.DataFrame, outcome: str, target: str):
 def _exhaustive_single(df: pd.DataFrame, outcome: str, target: str, **kwargs) -> List:
     try:
         rule = solve(df, outcome=outcome, target=target, **kwargs)
-    except NotEnoughPointsError:
+    except (NotEnoughPointsError, DataTransformationError):
         return []
 
     try:
@@ -157,7 +180,7 @@ def _exhaustive_best(df: pd.DataFrame, outcome: str, **kwargs) -> List:
                 best_target = target
             else:
                 continue
-        except NotEnoughPointsError:
+        except (NotEnoughPointsError, DataTransformationError):
             return []
 
         try:
@@ -180,7 +203,7 @@ def _exhaustive_any(df: pd.DataFrame, outcome: str, **kwargs) -> List:
         try:
             rule = solve(df, outcome=outcome, target=target, **kwargs)
             rule_set.append(rule)
-        except NotEnoughPointsError:
+        except (NotEnoughPointsError, DataTransformationError):
             return []
 
     try:
